
/* Portions Copyright (c) 1995,1996,1997 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




#ifndef DNSSEC_H
#define DNSSEC_H
/* $Header: /home/ogud/sec_bind/dnssec_lib/RCS/dnssec.h,v 1.8 1997/01/22 21:28:24 ogud Exp $ */
/* the following declarations are here for defensive programming reasons*/
#ifndef OK
# define OK     0
#endif
#ifndef NOT_OK
# define NOT_OK ~OK
#endif 
#ifndef INIT
# define INIT(a) 
#endif 
#ifndef DECL
# define DECL extern
#endif

typedef struct general_key {
  char        *key_name;
  void         *KEY_struct;
  int          key_size, /* this is the size of the key in bits */
               axfr,     /* is this key doing axfr calculations */
               proto,    /* what protocols this key can be used for */
               alg;      /* algorithm number from key record */
  u_int32_t    ttl;      /* ttl value of the databuf record */
  u_int16_t    foot,     /* footprint of the key */
               flags;    /* and the flags of the public key */
} KEY;

#ifdef Z_NIL /* this is only needed named is present */
struct in_data { /* data structure for putting data to be signed into */
	int             pre_len, dat_len;
	u_int32_t       ttl;         /* ttl value to use on this data set */
	u_char          prefix[1024];
	u_char          data[8192];  /* hope this is enough */
};

#ifdef SIGNER_APP
typedef struct nxt_struct {       /* simple sorted linked list */
  int len,max_type, max_sig;      /* length of name,  largest type numbers */
  char  *dname, *name, *signer;   /* dname name in zone,full domain name,
				     name of signer */
  struct nxt_struct *next;
  u_char nxt_rr[32],sign_rr[32];  /* large enough for 255 types */
} nxt;

DECL u_int32_t      signer_serial INIT(0);
DECL nxt            *nxt_start    INIT(NULL);
DECL int            hide          INIT(0);
DECL int            do_not_sign  INIT(0);
DECL char          *default_signer INIT(NULL);
#endif /* SIGNER_APP */

/* 
 * DNSSEC signatrue and verifiction functions
 */
DECL int            sign_rec     INIT(1);
DECL int            key_file     INIT(0);
DECL struct in_data sig_data;
DECL int            dp_rrcount   INIT(0); /* number of elements in dp_rrarray*/
DECL unsigned int   ttl_val;

int         verify_rec      __P(( char *, char *, struct databuf *, int)),
            do_signature    __P(( char *, short, struct databuf *, 
				 char *, KEY *));
void        sign_zone       __P(( struct zoneinfo *, char *,char *)),
            init_signer     __P(( KEY *));

/* 
 * Support functions for DNSSEC routines 
 */ 
void            put_rr      __P(( char *, struct databuf *, u_int32_t));
void            expand_rrs  __P(( char *));
void            append_rr   __P(( char *, struct databuf *, u_int32_t));
struct databuf *nt_lookup   __P(( const char *, short ,struct namebuf **));
struct databuf *nxt_check   __P(( char *, char *, struct databuf *, 
				  struct databuf *, struct databuf *));
#ifdef SIGNER_APP
/* 
 * NXT routines for signer 
 */ 
void           nxt_init     __P(( char *, char * ));
int            nxt_addname  __P(( char *, int, char *));
int            nxt_make     __P(( char *, u_int32_t, u_int16_t));
#endif /* SIGNER_APP */  
#endif /* Z_NIL */
DECL int            do_axfr      INIT(1);  /* Do this by default */

/* 
 * DNSSEC Crypto API defintions 
 */
int           dnssec_check     __P(( int));
void          dnssec_freekey   __P(( KEY *));
int           dnssec_genkey    __P(( char *, int, int, int, int, int));
KEY          *dnssec_getkey    __P(( char *, int));
KEY          *dnssec_pubkey    __P(( int, int, u_char *, u_char *));
KEY          *dnssec_readpubkey __P(( char *));
int           dnssec_sign      __P(( int, u_char *, int, KEY *, u_char *));
int           dnssec_verify    __P(( int, u_char *,int,KEY *,u_char *,int));
int           dnssec_writepubkey __P(( KEY *, int));
int           dnssec_keycmp     __P(( KEY *, KEY *));

/* DNSSEC_API control flags */
#define SIG_MODE_AXFR   8
#define SIG_MODE_INIT   1
#define SIG_MODE_UPDATE 2
#define SIG_MODE_FINAL  4
#define SIG_MODE_ALL    (SIG_MODE_INIT | SIG_MODE_UPDATE | SIG_MODE_FINAL)
#define DNSSEC_FORCE_READ  1
#define DNSSEC_STANDARD    0 

/* DNSSEC algorithm codes */
#define KEY_RSA                1
#define KEY_NULL             253
#define KEY_PRIVATE          254
#define KEY_EXPAND           255

#ifdef DNS_SECURITY
#ifndef DO_SOA
#define DO_SOA 1
#define NO_SOA 0
#endif

#define SIGN_TYPE   0
#define SIGN_ALG    2  /* 2 + SIGN_TYPE */
#define SIGN_LABELS 3  /* 1 + SIGN_ALG */
#define SIGN_OTTL   4  /* 1 + SIGN_LABELS */
#define SIGN_EXPIR  8  /* 4 + SIGN_OTTL */
#define SIGN_SIGNED 12 /* 4 + SIGN_EXPIR */
#define SIGN_FOOT   16 /* 4 + SIGN_SIGNED */
#define SIGN_SIGNER 18 /* 2 + SIGN_FOOT */
#endif 

/* error codes returned by various routines */
#define VERIFY_HARD_FAIL      -45
#define VERIFY_SOFT_FAIL      -46
#define SIGN_INIT_FAILURE     -23
#define SIGN_UPDATE_FAILURE   -24
#define SIGN_FINAL_FAILURE    -25
#define VERIFY_INIT_FAILURE   -26
#define VERIFY_UPDATE_FAILURE -27
#define VERIFY_FINAL_FAILURE  -28
#define UNKNOWN_KEYALG        -29

/* NXT error codes */
#define NXT_BAD_TYPE            1
#define NXT_NOT_IN_ZONE         2
#define NXT_SUBZONE_NAME        3

#define DO_NXT_LOOKUP           6 /* this should be in nameser.h */
#define SIGN_BITS 8
#define SIGN_BIT_SET(  n, p) (p[(n)/SIGN_BITS] |=  (0x80>>((n)%SIGN_BITS)))
#define SIGN_BIT_CLEAR(n, p) (p[(n)/SIGN_BITS] &= ~(0x80>>((n)%SIGN_BITS)))
#define SIGN_BIT_ISSET(n, p) (p[(n)/SIGN_BITS] &   (0x80>>((n)%SIGN_BITS)))
#define SIG_COVERS(dp) ((u_int16_t)(_getshort((char *) dp->d_data)))
/* I want memonic definitions to use in the des_function call */
#define DES_ENCRYPT             1 /*values happen to be the same as in RSAREF*/
#define DES_DECRYPT             0 
#endif /* DNSSEC_H */

